@@ -12,7 +12,7 @@ from thor.events import on
 from thor.http import HttpServer
 
 class TestHttpServer(framework.ClientServerTestCase):
-            
+
     def create_server(self, test_host, test_port, server_side):
         server = HttpServer(test_host, test_port, loop=self.loop)
         server_side(server)
@@ -34,7 +34,7 @@ class TestHttpServer(framework.ClientServerTestCase):
         expected, and verify that it actually happened.
         """
         exchange.test_happened = False
-        
+
         @on(exchange)
         def error(err_msg):
             exchange.test_happened = True
@@ -46,7 +46,7 @@ class TestHttpServer(framework.ClientServerTestCase):
             self.assertEqual(method, expected.get('method', method))
             self.assertEqual(uri, expected.get('phrase', uri))
 
-        exchange.tmp_req_body = ""
+        exchange.tmp_req_body = b""
         @on(exchange)
         def request_body(chunk):
             exchange.tmp_req_body += chunk
@@ -55,15 +55,15 @@ class TestHttpServer(framework.ClientServerTestCase):
         def request_done(trailers):
             exchange.test_happened = True
             self.assertEqual(
-                trailers, 
+                trailers,
                 expected.get('req_trailers', trailers)
             )
             self.assertEqual(
-                exchange.tmp_req_body, 
+                exchange.tmp_req_body,
                 expected.get('body', exchange.tmp_req_body)
             )
             self.loop.stop()
-            
+
         @on(self.loop)
         def stop():
             self.assertTrue(exchange.test_happened)
@@ -73,15 +73,15 @@ class TestHttpServer(framework.ClientServerTestCase):
         def server_side(server):
             def check(exchange):
                 self.check_exchange(exchange, {
-                    'method': 'GET',
-                    'uri': '/'                    
+                    'method': b'GET',
+                    'uri': b'/'
                 })
             server.on('exchange', check)
-            
+
         def client_side(client_conn):
-            client_conn.sendall("""\
+            client_conn.sendall(b"""\
 GET / HTTP/1.1
-Host: %s:%s
+Host: %s:%i
 
 """ % (framework.test_host, framework.test_port))
             time.sleep(1)
@@ -93,17 +93,17 @@ Host: %s:%s
         def server_side(server):
             def check(exchange):
                 self.check_exchange(exchange, {
-                    'method': 'GET',
-                    'uri': '/'
+                    'method': b'GET',
+                    'uri': b'/'
                 })
             server.on('exchange', check)
-            
-        def client_side(client_conn):
-            client_conn.sendall("""\
-            
-GET / HTTP/1.1
-Host: %s:%s
 
+        def client_side(client_conn):
+            client_conn.sendall(b"""\
+
+GET / HTTP/1.1
+Host: %s:%i\r
+\r
 """ % (framework.test_host, framework.test_port))
             time.sleep(1)
             client_conn.close()
@@ -114,15 +114,15 @@ Host: %s:%s
         def server_side(server):
             def check(exchange):
                 self.check_exchange(exchange, {
-                    'method': 'POST',
-                    'uri': '/foo'                    
+                    'method': b'POST',
+                    'uri': b'/foo'
                 })
             server.on('exchange', check)
-            
+
         def client_side(client_conn):
-            client_conn.sendall("""\
+            client_conn.sendall(b"""\
 POST / HTTP/1.1
-Host: %s:%s
+Host: %s:%i
 Content-Type: text/plain
 Content-Length: 5
 
@@ -130,21 +130,21 @@ Content-Length: 5
             time.sleep(1)
             client_conn.close()
         self.go([server_side], [client_side])
-        
+
 
     def test_post_extra_crlf(self):
         def server_side(server):
             def check(exchange):
                 self.check_exchange(exchange, {
-                    'method': 'POST',
-                    'uri': '/foo'                    
+                    'method': b'POST',
+                    'uri': b'/foo'
                 })
             server.on('exchange', check)
-            
+
         def client_side(client_conn):
-            client_conn.sendall("""\
+            client_conn.sendall(b"""\
 POST / HTTP/1.1
-Host: %s:%s
+Host: %s:%i
 Content-Type: text/plain
 Content-Length: 5
 
@@ -152,7 +152,7 @@ Content-Length: 5
 """ % (framework.test_host, framework.test_port))
             time.sleep(1)
             client_conn.close()
-        self.go([server_side], [client_side])        
+        self.go([server_side], [client_side])
 
 
 #    def test_pipeline(self):
@@ -168,14 +168,14 @@ Content-Length: 5
 #            @on(self.loop)
 #            def stop():
 #                self.assertEqual(server.ex_count, 2)
-#            
+#
 #        def client_side(client_conn):
 #            client_conn.sendall("""\
 #GET / HTTP/1.1
-#Host: %s:%s
+#Host: %s:%i
 #
 #GET / HTTP/1.1
-#Host: %s:%s
+#Host: %s:%i
 #
 #""" % (
 #    framework.test_host, framework.test_port,
@@ -198,6 +198,7 @@ Content-Length: 5
 #    def test_unknown_transfercode(self): # should be 501
 #    def test_shutdown(self):
 #    def test_alternate_tcp_server(self):
+#    def test_startline_encoding(self):
 
 
 if __name__ == '__main__':
