@@ -29,7 +29,7 @@ class TestUdpEndpoint(unittest.TestCase):
 
     def tearDown(self):
         self.ep1.shutdown()
-        
+
     def timeout(self):
         self.timeout_hit = True
         self.loop.stop()
@@ -39,29 +39,29 @@ class TestUdpEndpoint(unittest.TestCase):
 
     def output(self, msg):
         self.ep2.send(msg, test_host, test_port)
-        
+
     def test_basic(self):
-        self.loop.schedule(1, self.output, 'foo!')
-        self.loop.schedule(2, self.output, 'bar!')
-        
+        self.loop.schedule(1, self.output, b'foo!')
+        self.loop.schedule(2, self.output, b'bar!')
+
         def check():
-            self.assertEqual(self.datagrams[0][0], 'foo!')
-            self.assertEqual(self.datagrams[1][0], 'bar!')
+            self.assertEqual(self.datagrams[0][0], b'foo!')
+            self.assertEqual(self.datagrams[1][0], b'bar!')
             self.loop.stop()
         self.loop.schedule(3, check)
         self.loop.run()
-        
+
     def test_bigdata(self):
-        self.loop.schedule(1, self.output, 'a' * 100)
-        self.loop.schedule(2, self.output, 'b' * 1000)
-        self.loop.schedule(3, self.output, 'c' * self.ep1.max_dgram)
-        
+        self.loop.schedule(1, self.output, b'a' * 100)
+        self.loop.schedule(2, self.output, b'b' * 1000)
+        self.loop.schedule(3, self.output, b'c' * self.ep1.max_dgram)
+
         def check():
-            self.assertEqual(self.datagrams[0][0], 'a' * 100)
-            self.assertEqual(self.datagrams[1][0], 'b' * 1000)
-            # we only check the first 1000 characters because, well, 
+            self.assertEqual(self.datagrams[0][0], b'a' * 100)
+            self.assertEqual(self.datagrams[1][0], b'b' * 1000)
+            # we only check the first 1000 characters because, well,
             # it's lossy.
-            self.assertEqual(self.datagrams[2][0][:1000], 'c' * 1000)
+            self.assertEqual(self.datagrams[2][0][:1000], b'c' * 1000)
             self.loop.stop()
         self.loop.schedule(4, check)
         self.loop.run()
