@@ -4,41 +4,16 @@
 Asynchronous event loops
 
 This is a generic library for building asynchronous event loops, using
-Python 2.6+'s built-in poll / epoll / kqueue support.
+Python's built-in poll / epoll / kqueue support.
 """
 
-__author__ = "Mark Nottingham <mnot@mnot.net>"
-__copyright__ = """\
-Copyright (c) 2005-2013 Mark Nottingham
-
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-THE SOFTWARE.
-"""
-
-import bisect
 import select
 import sys
 import time as systime
+from typing import Callable, List, Dict, Set, Iterable, Tuple, Any # pylint: disable=unused-import
 
 from thor.events import EventEmitter
 
-assert sys.version_info[0] == 2 and sys.version_info[1] >= 6, \
-    "Please use Python 2.6 or greater"
 
 __all__ = ['run', 'stop', 'schedule', 'time', 'running', 'debug']
 
@@ -51,13 +26,13 @@ class EventSource(EventEmitter):
     An instance should map to one thing with an interesting file
     descriptor, registered with register_fd.
     """
-    def __init__(self, loop=None):
+    def __init__(self, loop: 'LoopBase' = None) -> None:
         EventEmitter.__init__(self)
         self._loop = loop or _loop
-        self._interesting_events = set()
-        self._fd = None
+        self._interesting_events = set()  # type: Set[str]
+        self._fd = None  # type: int
 
-    def register_fd(self, fd, event=None):
+    def register_fd(self, fd: int, event: str = None) -> None:
         """
         Register myself with the loop using file descriptor fd.
         If event is specified, start emitting it.
@@ -66,47 +41,55 @@ class EventSource(EventEmitter):
         self._loop.register_fd(self._fd, [], self)
         self.event_add(event)
 
-    def unregister_fd(self):
+    def unregister_fd(self) -> None:
         "Unregister myself from the loop."
         if self._fd:
             self._loop.unregister_fd(self._fd)
             self._fd = None
 
-    def event_add(self, event):
+    def event_add(self, event: str) -> None:
         "Start emitting the given event."
         if event and event not in self._interesting_events:
             self._interesting_events.add(event)
             self._loop.event_add(self._fd, event)
 
-    def event_del(self, event):
+    def event_del(self, event: str) -> None:
         "Stop emitting the given event."
         if event in self._interesting_events:
             self._interesting_events.remove(event)
             self._loop.event_del(self._fd, event)
 
+    def interesting_events(self) -> Set[str]:
+        return self._interesting_events
+
 
 class LoopBase(EventEmitter):
     """
     Base class for async loops.
     """
-    _event_types = {} # map of event types to names; override.
+    _event_types = {}   # type: Dict[int, str] # map of event types to names; override.
 
-    def __init__(self, precision=None):
+    def __init__(self, precision: float = None) -> None:
         EventEmitter.__init__(self)
         self.precision = precision or .5 # of running scheduled queue (secs)
-        self.running = False # whether or not the loop is running (read-only)
-        self.__sched_events = []
-        self._fd_targets = {}
-        self.__now = None
-        self._eventlookup = dict(
-            [(v,k) for (k,v) in self._event_types.items()]
-        )
-        self.__event_cache = {}
+        self.running = False      # whether or not the loop is running (read-only)
+        self.__sched_events = []  # type: List[Tuple[float, Callable]]
+        self._fd_targets = {}     # type: Dict[int, EventSource]
+        self.__now = None         # type: float
+        self._eventlookup = dict([(v, k) for (k, v) in self._event_types.items()])
+        self.__event_cache = {}   # type: Dict[int, Set[str]]
 
-    def run(self):
+    def __repr__(self) -> str:
+        name = self.__class__.__name__
+        is_running = 'running' if self.running else 'not-running'
+        events = len(self.__sched_events)
+        targets = len(self._fd_targets)
+        return "<%s - %s, %i sched_events, %i fd_targets>" % (name, is_running, events, targets)
+
+    def run(self) -> None:
         "Start the loop."
         self.running = True
-        last_event_check = 0
+        last_event_check = 0  # type: float
         self.__now = systime.time()
         self.emit('start')
         while self.running:
@@ -117,27 +100,20 @@ class LoopBase(EventEmitter):
             if debug:
                 delay = self.__now - fd_start
                 if delay >= self.precision * 1.5:
-                    sys.stderr.write(
-                     "WARNING: long fd delay (%.2f)\n" % delay
-                    )
+                    sys.stderr.write("WARNING: long fd delay (%.2f)\n" % delay)
             # find scheduled events
-            if not self.running:
-                break
             delay = self.__now - last_event_check
             if delay >= self.precision * 0.90:
                 if debug:
                     if last_event_check and (delay >= self.precision * 4):
-                        sys.stderr.write(
-                          "WARNING: long loop delay (%.2f)\n" % delay
-                        )
+                        sys.stderr.write("WARNING: long loop delay (%.2f)\n" % delay)
                     if len(self.__sched_events) > 5000:
-                        sys.stderr.write(
-                          "WARNING: %i events scheduled\n" % \
-                            len(self.__sched_events))
+                        sys.stderr.write("WARNING: %i events scheduled\n" % \
+                                         len(self.__sched_events))
                 last_event_check = self.__now
                 for event in self.__sched_events:
                     when, what = event
-                    if self.__now >= when:
+                    if self.running and self.__now >= when:
                         try:
                             self.__sched_events.remove(event)
                         except ValueError:
@@ -149,92 +125,99 @@ class LoopBase(EventEmitter):
                         if debug:
                             delay = systime.time() - ev_start
                             if delay > self.precision * 2:
-                                sys.stderr.write(
-                        "WARNING: long event delay (%.2f): %s\n" % \
-                                (delay, repr(what)) 
-                                )
+                                sys.stderr.write("WARNING: long event delay (%.2f): %s\n" % \
+                                                 (delay, what.__name__))
                     else:
                         break
 
-    def _run_fd_events(self):
+    def _run_fd_events(self) -> None:
         "Run loop-specific FD events."
         raise NotImplementedError
 
-    def stop(self):
+    def stop(self) -> None:
         "Stop the loop and unregister all fds."
         self.__sched_events = []
         self.__now = None
         self.running = False
-        for fd in self._fd_targets.keys():
+        for fd in list(self._fd_targets):
             self.unregister_fd(fd)
         self.emit('stop')
 
-    def register_fd(self, fd, events, target):
+    def register_fd(self, fd: int, events: List[str], target: EventSource) -> None:
         "emit events on target when they occur on fd."
         raise NotImplementedError
 
-    def unregister_fd(self, fd):
+    def unregister_fd(self, fd: int) -> None:
         "Stop emitting events from fd."
         raise NotImplementedError
 
-    def fd_count(self):
+    def fd_count(self) -> int:
         "Return how many FDs are currently monitored by the loop."
         return len(self._fd_targets)
 
-    def event_add(self, fd, event):
+    def event_add(self, fd: int, event: str) -> None:
         "Start emitting event for fd."
         raise NotImplementedError
 
-    def event_del(self, fd, event):
+    def event_del(self, fd: int, event: str) -> None:
         "Stop emitting event for fd"
         raise NotImplementedError
 
-    def _fd_event(self, event, fd):
+    def _fd_event(self, event: str, fd: int) -> None:
         "An event has occured on an fd."
-        if self._fd_targets.has_key(fd):
+        if fd in self._fd_targets:
             self._fd_targets[fd].emit(event)
         # TODO: automatic unregister on 'close'?
 
-    def time(self):
+    def time(self) -> float:
         "Return the current time (to avoid a system call)."
         return self.__now or systime.time()
 
-    def schedule(self, delta, callback, *args):
+    def schedule(self, delta: float, callback: Callable, *args: Any) -> 'ScheduledEvent':
         """
         Schedule callable callback to be run in delta seconds with *args.
 
         Returns an object which can be used to later remove the event, by
         calling its delete() method.
         """
-        def cb():
+        def cb() -> None: # FIXME: can't compare functions in py3. Suck.
             if callback:
                 callback(*args)
         cb.__name__ = callback.__name__
         new_event = (self.time() + delta, cb)
         events = self.__sched_events
-        bisect.insort(events, new_event)
-        class event_holder:
-            def __init__(self):
-                self._deleted = False
-            def delete(self):
-                if not self._deleted:
-                    try:
-                        events.remove(new_event)
-                        self._deleted = True
-                    except ValueError: # already gone
-                        pass
-        return event_holder()
+        self._insort(events, new_event)
+        return ScheduledEvent(self, new_event)
 
-    def _eventmask(self, events):
+    def schedule_del(self, event: Tuple[float, Callable]) -> None:
+        try:
+            self.__sched_events.remove(event)
+        except ValueError: # already gone
+            pass
+
+    @staticmethod
+    def _insort(a: List, x: Any, lo: int = 0, hi: int = None) -> None:
+        if lo < 0:
+            raise ValueError('lo must be non-negative')
+        if hi is None:
+            hi = len(a)
+        while lo < hi:
+            mid = (lo+hi)//2
+            if x[0] < a[mid][0]:
+                hi = mid
+            else: lo = mid+1
+        a.insert(lo, x)
+
+    def _eventmask(self, events: Iterable[str]) -> int:
         "Calculate the mask for a list of events."
         eventmask = 0
         for event in events:
             eventmask |= self._eventlookup.get(event, 0)
         return eventmask
 
-    def _filter2events(self, evfilter):
+    def _filter2events(self, evfilter: int) -> Set[str]:
         "Calculate the events implied by a given filter."
-        if not self.__event_cache.has_key(evfilter):
+        if evfilter not in self.__event_cache:
             events = set()
             for et in self._event_types:
                 if et & evfilter:
@@ -243,41 +226,56 @@ class LoopBase(EventEmitter):
         return self.__event_cache[evfilter]
 
 
+class ScheduledEvent:
+    """
+    Holds a scheduled event.
+    """
+    def __init__(self, loop: LoopBase, event: Tuple[float, Callable]) -> None:
+        self._loop = loop
+        self._event = event
+        self._deleted = False
+
+    def delete(self) -> None:
+        if not self._deleted:
+            self._loop.schedule_del(self._event)
+            self._deleted = True
+
+
 class PollLoop(LoopBase):
     """
     A poll()-based async loop.
     """
 
-    def __init__(self, *args):
+    def __init__(self, *args: Any) -> None:
         # pylint: disable=E1101
         self._event_types = {
-            select.POLLIN: 'readable',
-            select.POLLOUT: 'writable',
-            select.POLLERR: 'error',
-            select.POLLHUP: 'close',
+            select.POLLIN: 'fd_readable',
+            select.POLLOUT: 'fd_writable',
+            select.POLLERR: 'fd_error',
+            select.POLLHUP: 'fd_close'}
     #        select.POLLNVAL - TODO
-        }
+
         LoopBase.__init__(self, *args)
         self._poll = select.poll()
         # pylint: enable=E1101
 
-    def register_fd(self, fd, events, target):
+    def register_fd(self, fd: int, events: List[str], target: EventSource) -> None:
         self._fd_targets[fd] = target
         self._poll.register(fd, self._eventmask(events))
 
-    def unregister_fd(self, fd):
+    def unregister_fd(self, fd: int) -> None:
         self._poll.unregister(fd)
         del self._fd_targets[fd]
 
-    def event_add(self, fd, event):
-        eventmask = self._eventmask(self._fd_targets[fd]._interesting_events)
+    def event_add(self, fd: int, event: str) -> None:
+        eventmask = self._eventmask(self._fd_targets[fd].interesting_events())
         self._poll.register(fd, eventmask)
 
-    def event_del(self, fd, event):
-        eventmask = self._eventmask(self._fd_targets[fd]._interesting_events)
+    def event_del(self, fd: int, event: str) -> None:
+        eventmask = self._eventmask(self._fd_targets[fd].interesting_events())
         self._poll.register(fd, eventmask)
 
-    def _run_fd_events(self):
+    def _run_fd_events(self) -> None:
         event_list = self._poll.poll(self.precision)
         for fileno, eventmask in event_list:
             for event in self._filter2events(eventmask):
@@ -289,19 +287,19 @@ class EpollLoop(LoopBase):
     An epoll()-based async loop.
     """
 
-    def __init__(self, *args):
+    def __init__(self, *args: Any) -> None:
         # pylint: disable=E1101
         self._event_types = {
-            select.EPOLLIN: 'readable',
-            select.EPOLLOUT: 'writable',
-            select.EPOLLHUP: 'close',
-            select.EPOLLERR: 'error'
+            select.EPOLLIN: 'fd_readable',
+            select.EPOLLOUT: 'fd_writable',
+            select.EPOLLHUP: 'fd_close',
+            select.EPOLLERR: 'fd_error'
         }
         LoopBase.__init__(self, *args)
         self._epoll = select.epoll()
         # pylint: enable=E1101
 
-    def register_fd(self, fd, events, target):
+    def register_fd(self, fd: int, events: List[str], target: EventSource) -> None:
         eventmask = self._eventmask(events)
         if fd in self._fd_targets:
             self._epoll.modify(fd, eventmask)
@@ -309,24 +307,22 @@ class EpollLoop(LoopBase):
             self._fd_targets[fd] = target
             self._epoll.register(fd, eventmask)
 
-    def unregister_fd(self, fd):
+    def unregister_fd(self, fd: int) -> None:
         self._epoll.unregister(fd)
         del self._fd_targets[fd]
 
-    def event_add(self, fd, event):
-        eventmask = self._eventmask(self._fd_targets[fd]._interesting_events)
+    def event_add(self, fd: int, event: str) -> None:
+        eventmask = self._eventmask(self._fd_targets[fd].interesting_events())
         self._epoll.modify(fd, eventmask)
 
-    def event_del(self, fd, event):
+    def event_del(self, fd: int, event: str) -> None:
         try:
-            eventmask = self._eventmask(
-                self._fd_targets[fd]._interesting_events
-            )
+            eventmask = self._eventmask(self._fd_targets[fd].interesting_events())
         except KeyError:
             return # no longer interested
         self._epoll.modify(fd, eventmask)
 
-    def _run_fd_events(self):
+    def _run_fd_events(self) -> None:
         event_list = self._epoll.poll(self.precision)
         for fileno, eventmask in event_list:
             for event in self._filter2events(eventmask):
@@ -337,53 +333,50 @@ class KqueueLoop(LoopBase):
     """
     A kqueue()-based async loop.
     """
-    def __init__(self, *args):
+    def __init__(self, *args: Any) -> None:
         self._event_types = {
-            select.KQ_FILTER_READ: 'readable',
-            select.KQ_FILTER_WRITE: 'writable'
-        }
+            select.KQ_FILTER_READ: 'fd_readable',
+            select.KQ_FILTER_WRITE: 'fd_writable'}
         LoopBase.__init__(self, *args)
         self.max_ev = 50 # maximum number of events to pull from the queue
         self._kq = select.kqueue()
 
     # TODO: override schedule() to use kqueue event scheduling.
 
-    def register_fd(self, fd, events, target):
+    def register_fd(self, fd: int, events: List[str], target: EventSource) -> None:
         self._fd_targets[fd] = target
         for event in events:
             self.event_add(fd, event)
 
-    def unregister_fd(self, fd):
+    def unregister_fd(self, fd: int) -> None:
         try:
             obj = self._fd_targets[fd]
         except KeyError:
             return
-        for event in list(obj._interesting_events):
+        for event in list(obj.interesting_events()):
             obj.event_del(event)
         del self._fd_targets[fd]
 
-    def event_add(self, fd, event):
+    def event_add(self, fd: int, event: str) -> None:
         eventmask = self._eventmask([event])
         if eventmask:
-            ev = select.kevent(fd, eventmask,
-                select.KQ_EV_ADD | select.KQ_EV_ENABLE
-            )
+            ev = select.kevent(fd, eventmask, select.KQ_EV_ADD | select.KQ_EV_ENABLE)
             self._kq.control([ev], 0, 0)
 
-    def event_del(self, fd, event):
+    def event_del(self, fd: int, event: str) -> None:
         eventmask = self._eventmask([event])
         if eventmask:
             ev = select.kevent(fd, eventmask, select.KQ_EV_DELETE)
             self._kq.control([ev], 0, 0)
 
-    def _run_fd_events(self):
+    def _run_fd_events(self) -> None:
         events = self._kq.control([], self.max_ev, self.precision)
         for e in events:
             event_types = self._filter2events(e.filter)
             for event_type in event_types:
                 self._fd_event(event_type, int(e.ident))
             if e.flags & select.KQ_EV_EOF:
-                self._fd_event('close', int(e.ident))
+                self._fd_event('fd_close', int(e.ident))
             if e.flags & select.KQ_EV_ERROR:
                 pass
             # TODO: pull errors, etc. out of flags and fflags
@@ -395,7 +388,7 @@ class KqueueLoop(LoopBase):
     		#	buffer.
 
 
-def make(precision=None):
+def make(precision: float = None) -> LoopBase:
     """
     Create and return a named loop that is suitable for the current system. If
     _precision_ is given, it indicates how often scheduled events will be run.
@@ -403,6 +396,7 @@ def make(precision=None):
     Returned loop instances have all of the methods and instance variables
     that *thor.loop* has.
     """
+    loop = None # type: LoopBase
     if hasattr(select, 'epoll'):
         loop = EpollLoop(precision)
     elif hasattr(select, 'kqueue'):
@@ -410,8 +404,7 @@ def make(precision=None):
     elif hasattr(select, 'poll'):
         loop = PollLoop(precision)
     else:
-        # TODO: select()-based loop (I suppose)
-        raise ImportError, "What is this thing, a Windows box?"
+        raise ImportError("What is this thing, a Windows box?")
     return loop
 
 _loop = make() # by default, just one big loop.
@@ -420,4 +413,4 @@ stop = _loop.stop
 schedule = _loop.schedule
 time = _loop.time
 running = _loop.running
-debug = False
\ No newline at end of file
+debug = False
