@@ -11,35 +11,15 @@ you (using a 'data' event), and you push data to the network connection
 (using the write method).
 """
 
-__author__ = "Mark Nottingham <mnot@mnot.net>"
-__copyright__ = """\
-Copyright (c) 2005-2013 Mark Nottingham
-
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-THE SOFTWARE.
-"""
-
 import errno
 import os
 import sys
 import socket
+from typing import Tuple, List, Union, Type # pylint: disable=unused-import
+import ssl as sys_ssl # pylint: disable=unused-import
 
-from thor.loop import EventSource, schedule
+from thor.loop import EventSource, LoopBase, schedule
+from thor.loop import ScheduledEvent # pylint: disable=unused-import
 
 
 class TcpConnection(EventSource):
@@ -106,16 +86,13 @@ class TcpConnection(EventSource):
     write_bufsize = 16
     read_bufsize = 1024 * 16
 
-    _block_errs = set([(socket.error, e) for e in [
-        errno.EAGAIN, errno.EWOULDBLOCK, errno.ETIMEDOUT
-    ]])
-    _close_errs = set([(socket.error, e) for e in [
+    block_errs = set([errno.EAGAIN, errno.EWOULDBLOCK, errno.ETIMEDOUT])
+    close_errs = set([
         errno.EBADF, errno.ECONNRESET, errno.ESHUTDOWN,
         errno.ECONNABORTED, errno.ECONNREFUSED,
-        errno.ENOTCONN, errno.EPIPE
-    ]])
+        errno.ENOTCONN, errno.EPIPE])
 
-    def __init__(self, sock, host, port, loop=None):
+    def __init__(self, sock: socket.socket, host: bytes, port: int, loop: LoopBase = None) -> None:
         EventSource.__init__(self, loop)
         self.socket = sock
         self.host = host
@@ -124,14 +101,14 @@ class TcpConnection(EventSource):
         self._input_paused = True # we start with input paused
         self._output_paused = False
         self._closing = False
-        self._write_buffer = []
+        self._write_buffer = []   # type: List[bytes]
 
         self.register_fd(sock.fileno())
-        self.on('readable', self.handle_read)
-        self.on('writable', self.handle_write)
-        self.on('close', self.handle_close)
+        self.on('fd_readable', self.handle_readable)
+        self.on('fd_writable', self.handle_writable)
+        self.on('fd_close', self._handle_close)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         status = [self.__class__.__module__ + "." + self.__class__.__name__]
         status.append(self.tcp_connected and 'connected' or 'disconnected')
         status.append('%s:%s' % (self.host, self.port))
@@ -145,94 +122,88 @@ class TcpConnection(EventSource):
             status.append('%s write buffered' % len(self._write_buffer))
         return "<%s at %#x>" % (", ".join(status), id(self))
 
-    def handle_read(self):
+    def handle_readable(self) -> None:
         "The connection has data read for reading"
         try:
-            # TODO: look into recv_into (but see python issue7827)
             data = self.socket.recv(self.read_bufsize)
-        except Exception, why:
-            err = (type(why), why[0])
-            if err in self._block_errs:
+        except (socket.error, OSError) as why:
+            if why.args[0] in self.block_errs:
                 return
-            elif err in self._close_errs:
-                self.emit('close')
+            if why.args[0] in self.close_errs:
+                self._handle_close()
                 return
-            else:
-                raise
-        if data == "":
-            self.emit('close')
+            raise
+        if data == b"":
+            self._handle_close()
         else:
             self.emit('data', data)
 
-    # TODO: try using buffer; see
-    # http://itamarst.org/writings/pycon05/fast.html
-    def handle_write(self):
+    def handle_writable(self) -> None:
         "The connection is ready for writing; write any buffered data."
-        if len(self._write_buffer) > 0:
-            data = "".join(self._write_buffer)
+        if self._write_buffer:
+            data = b"".join(self._write_buffer)
             try:
                 sent = self.socket.send(data)
-            except Exception, why:
-                err = (type(why), why[0])
-                if err in self._block_errs:
+            except (socket.error, OSError) as why:
+                if why.args[0] in self.block_errs:
                     return
-                elif err in self._close_errs:
-                    self.emit('close')
+                if why.args[0] in self.close_errs:
+                    self._handle_close()
                     return
-                else:
-                    raise
+                raise
             if sent < len(data):
                 self._write_buffer = [data[sent:]]
             else:
                 self._write_buffer = []
-        if self._output_paused and \
-          len(self._write_buffer) < self.write_bufsize:
+        if self._output_paused and len(self._write_buffer) < self.write_bufsize:
             self._output_paused = False
             self.emit('pause', False)
         if self._closing:
-            self.close()
-        if len(self._write_buffer) == 0:
-            self.event_del('writable')
+            self._close()
+        if not self._write_buffer:
+            self.event_del('fd_writable')
 
-    def handle_close(self):
-        """
-        The connection has been closed by the other side.
-        """
-        self.tcp_connected = False
-        # TODO: make sure removing close doesn't cause problems.
-        self.removeListeners('readable', 'writable', 'close')
-        self.unregister_fd()
-        self.socket.close()
-
-    def write(self, data):
+    def write(self, data: bytes) -> None:
         "Write data to the connection."
         self._write_buffer.append(data)
         if len(self._write_buffer) > self.write_bufsize:
             self._output_paused = True
             self.emit('pause', True)
-        self.event_add('writable')
+        self.event_add('fd_writable')
 
-    def pause(self, paused):
+    def pause(self, paused: bool) -> None:
         """
         Temporarily stop/start reading from the connection and pushing
         it to the app.
         """
         if paused:
-            self.event_del('readable')
+            self.event_del('fd_readable')
         else:
-            self.event_add('readable')
+            self.event_add('fd_readable')
         self._input_paused = paused
 
-    def close(self):
+    def close(self) -> None:
         "Flush buffered data (if any) and close the connection."
         self.pause(True)
-        if len(self._write_buffer) > 0:
+        if self._write_buffer:
             self._closing = True
         else:
-            self.handle_close()
-
+            self._close()
         # TODO: should loop stop automatically close all conns?
 
+    def _handle_close(self) -> None:
+        "The connection has been closed by the other side."
+        self._close()
+        self.emit('close')
+
+    def _close(self) -> None:
+        self.tcp_connected = False
+        self.removeListeners('fd_readable', 'fd_writable', 'fd_close')
+        self.unregister_fd()
+        if self.socket:
+            self.socket.close()
+
+
 class TcpServer(EventSource):
     """
     An asynchronous TCP server.
@@ -247,16 +218,17 @@ class TcpServer(EventSource):
 
     conn_handler is called every time a new client connects.
     """
-    def __init__(self, host, port, sock=None, loop=None):
+    def __init__(self, host: bytes, port: int, sock: socket.socket = None,
+                 loop: LoopBase = None) -> None:
         EventSource.__init__(self, loop)
         self.host = host
         self.port = port
         self.sock = sock or server_listen(host, port)
-        self.on('readable', self.handle_accept)
-        self.register_fd(self.sock.fileno(), 'readable')
+        self.on('fd_readable', self.handle_accept)
+        self.register_fd(self.sock.fileno(), 'fd_readable')
         schedule(0, self.emit, 'start')
 
-    def handle_accept(self):
+    def handle_accept(self) -> None:
         try:
             conn, addr = self.sock.accept()
         except (TypeError, IndexError):
@@ -269,15 +241,15 @@ class TcpServer(EventSource):
 
     # TODO: should loop stop close listening sockets?
 
-    def shutdown(self):
+    def shutdown(self) -> None:
         "Stop accepting requests and close the listening socket."
-        self.removeListeners('readable')
+        self.removeListeners('fd_readable')
         self.sock.close()
         self.emit('stop')
         # TODO: emit close?
 
 
-def server_listen(host, port, backlog=None):
+def server_listen(host: bytes, port: int, backlog: int = None) -> socket.socket:
     "Return a socket listening to host:port."
     # TODO: IPV6
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -295,8 +267,8 @@ class TcpClient(EventSource):
     Emits:
       - connect (tcp_conn): upon connection
       - connect_error (err_type, err_id, err_str): if there's a problem
-        before getting a connection. err_type is socket.error or
-        socket.gaierror; err_id is the specific error encountered, and
+        before getting a connection. err_type is 'socket' or
+        'gai'; err_id is the specific error encountered, and
         err_str is its textual description.
 
     To connect to a server:
@@ -309,21 +281,19 @@ class TcpClient(EventSource):
     conn_handler will be called with the tcp_conn as the argument
     when the connection is made.
     """
-    def __init__(self, loop=None):
+    def __init__(self, loop: LoopBase = None) -> None:
         EventSource.__init__(self, loop)
-        self.host = None
-        self.port = None
-        self._timeout_ev = None
+        self.host = None  # type: bytes
+        self.port = None  # type: int
+        self._timeout_ev = None  # type: ScheduledEvent
         self._error_sent = False
-        # TODO: IPV6
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.sock.setblocking(False)
-        self.on('error', self.handle_conn_error)
-        self.register_fd(self.sock.fileno(), 'writable')
-        self.event_add('error')
+        self.on('fd_error', self.handle_fd_error)
+        self.register_fd(self.sock.fileno(), 'fd_writable')
+        self.event_add('fd_error')
 
-
-    def connect(self, host, port, connect_timeout=None):
+    def connect(self, host: bytes, port: int, connect_timeout: float = None) -> None:
         """
         Connect to host:port (with an optional connect timeout)
         and emit 'connect' when connected, or 'connect_error' in
@@ -331,29 +301,27 @@ class TcpClient(EventSource):
         """
         self.host = host
         self.port = port
-        self.on('writable', self.handle_connect)
+        self.on('fd_writable', self.handle_connect)
         # TODO: use socket.getaddrinfo(); needs to be non-blocking.
         try:
             err = self.sock.connect_ex((host, port))
-        except socket.gaierror, why:
-            self.handle_conn_error(socket.gaierror, why)
+        except socket.gaierror as why:
+            self.handle_socket_error(why, 'gai')
             return
-        except socket.error, why:
-            self.handle_conn_error(socket.error, why)
+        except socket.error as why:
+            self.handle_socket_error(why)
             return
         if err != errno.EINPROGRESS:
-            self.handle_conn_error(socket.error, [err, os.strerror(err)])
+            self.handle_socket_error(socket.error(err, os.strerror(err)))
             return
         if connect_timeout:
             self._timeout_ev = self._loop.schedule(
                 connect_timeout,
-                self.handle_conn_error,
-                socket.error,
-                [errno.ETIMEDOUT, os.strerror(errno.ETIMEDOUT)],
-                True
+                self.handle_socket_error,
+                socket.error(errno.ETIMEDOUT, os.strerror(errno.ETIMEDOUT))
             )
 
-    def handle_connect(self):
+    def handle_connect(self) -> None:
         self.unregister_fd()
         if self._timeout_ev:
             self._timeout_ev.delete()
@@ -361,32 +329,31 @@ class TcpClient(EventSource):
             return
         err = self.sock.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
         if err:
-            self.handle_conn_error(socket.error, [err, os.strerror(err)])
+            self.handle_socket_error(socket.error(err, os.strerror(err)))
         else:
-            tcp_conn = TcpConnection(
-                self.sock, self.host, self.port, self._loop
-            )
+            tcp_conn = TcpConnection(self.sock, self.host, self.port, self._loop)
             self.emit('connect', tcp_conn)
 
-    def handle_conn_error(self, err_type=None, why=None, close=False):
+    def handle_fd_error(self) -> None:
+        err_id = self.sock.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
+        err_str = os.strerror(err_id)
+        self.handle_conn_error('socket', err_id, err_str)
+
+    def handle_socket_error(self, why: Union[socket.error, socket.gaierror, sys_ssl.SSLError],
+                            err_type: str = "socket") -> None:
+        err_id = why.args[0]
+        err_str = why.args[1]
+        self.handle_conn_error(err_type, err_id, err_str)
+
+    def handle_conn_error(self, err_type: str, err_id: int, err_str: str,
+                          close: bool = True) -> None:
         """
         Handle a connect error.
-
-        @err_type - e.g., socket.error; defaults to socket.error
-        @why - tuple of [err_id, err_str]
-        @close - whether the error means the socket should be closed
         """
         if self._timeout_ev:
             self._timeout_ev.delete()
         if self._error_sent:
             return
-        if err_type is None:
-            err_type = socket.error
-            err_id = self.sock.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
-            err_str = os.strerror(err_id)
-        else:
-            err_id = why[0]
-            err_str = why[1]
         self._error_sent = True
         self.unregister_fd()
         self.emit('connect_error', err_type, err_id, err_str)
@@ -397,16 +364,14 @@ class TcpClient(EventSource):
 if __name__ == "__main__":
     # quick demo server
     from thor.loop import run, stop
-    server = TcpServer('localhost', int(sys.argv[-1]))
-    def handle_conn(conn):
+    server = TcpServer(b'localhost', int(sys.argv[-1]))
+    def handle_conn(conn: TcpConnection) -> None:
         conn.pause(False)
-        def echo(chunk):
+        def echo(chunk: bytes) -> None:
             if chunk.strip().lower() in ['quit', 'stop']:
                 stop()
             else:
-                conn.write("-> %s" % chunk)
+                conn.write(b"-> %s" % chunk)
         conn.on('data', echo)
     server.on('connect', handle_conn)
     run()
-
-
